﻿package com.actionsnippet.qbox.objects{
	
	import flash.display.*
	import flash.events.*;
	
	import com.actionsnippet.qbox.*
	
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.Joints.*;
	
	/**
	JointObject is a subclass of {@link com.actionsnippet.qbox.QuickObject} and should only be instantiated with the {@link com.actionsnippet.qbox.QuickBox2D#addJoint()} method.
	
	@author Zevan Rosser
	@version alpha 105
	*/
	public class JointObject extends QuickObject {
				   
		private var jointDef:b2DistanceJointDef
		public function JointObject(qbox:QuickBox2D, params:Object){
			super(qbox, params);
		}
		
		override public function get userData():*{
			return jointDef.userData;
		}
		
		override public function set userData(object:*):*{
			jointDef.userData = object;
		}
		
		override protected function defaultParams(p:Object):void{
			p.a = null;
			p.b = null;
			//p.vecA = new b2Vec2(0, 0);
			//p.vecB = new b2Vec2(1, 1);
			p.frequencyHz = 0;
			p.dampingRatio = 0;
			p.length = null;
			p.collideConnected = true;
		}
		
		override protected function build():void{
			var p:Object = params;
			
			if (p.a == null || p.b == null){
				throw new Error("your jointDef must have b2Body instances for its a and b properties")
			}
			if (p.vecA == null){
				p.vecA = new b2Vec2(p.x1, p.y1);
			}
			if (p.vecB == null){
				p.vecB = new b2Vec2(p.x2, p.y2);
			}
			
			 jointDef = new b2DistanceJointDef();
			
			 if (p.skin){
				userData = new p.skin();
				if (!(userData is MovieClip)){
					throw new Error("Sorry, joint skins must be dynamic. If your using a Sprite, just use a MovieClip instead.");
				}
				userData.startWidth = userData.width;
				userData.startHeight = userData.height;
				userData.p = params;
			 }else{
				userData = params; 
			 }
			
			 jointDef.Initialize(p.a, p.b, p.vecA, p.vecB);
			 jointDef.frequencyHz = p.frequencyHz;
			 jointDef.dampingRatio = p.dampingRatio;
			 if (p.length){
				jointDef.length = p.length;  
			 }
			 jointDef.collideConnected = p.collideConnected;
			 joint = w.CreateJoint(jointDef);
		}
	}
}